#!/usr/bin/env python
#
# A simple "trick-mode" looping player.
# Version 2, based on "about-to-finish" handling.
#
# Not working with speed rates other that 1.
#
# Get a test sample with:
# youtube-dl -t http://www.youtube.com/watch?v=yWa-YXiSk2Y

import sys

import gi
gi.require_version('Gst', '1.0')
from gi.repository import Gst
Gst.init(None)

from gi.repository import GObject
GObject.threads_init()


class Player:
    def __init__(self, uri, rate):
        self._uri = uri
        self._rate = rate

        self._player = Gst.ElementFactory.make("playbin", "player")
        self._player.set_property("uri", uri)
        self._player.connect("about-to-finish", self.on_about_to_finish)

        bus = self._player.get_bus()
        bus.add_signal_watch()
        bus.connect('message::error', self.on_error)
        bus.connect('message::state-changed', self.on_state_changed)

    def run(self):
        self._player.set_state(Gst.State.PLAYING)
        self.loop = GObject.MainLoop()
        self.loop.run()

    def quit(self):
        self._player.set_state(Gst.State.NULL)
        self.loop.quit()

    def on_about_to_finish(self, player):
        sys.stderr.write(".")
        self._player.set_property("uri", self._uri)

        # XXX After the first iteration of the loop the playback rate is
        # reset to normal. Do I need to re-set the rate here?

        # If I uncomment the following line, nothing changes:
        #self.set_rate(self._rate)

        # And nothing changes either if I set the seek_event manually using
        # position=0:
        #seek_event = Gst.Event.new_seek(self._rate,
        #                                Gst.Format.TIME,
        #                                Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
        #                                Gst.SeekType.SET, 0,
        #                                Gst.SeekType.NONE, 0)
        #self._player.send_event(seek_event)

    def on_error(self, bus, msg):
        (err, debug) = msg.parse_error()
        print "Error: %s" % err, debug
        self.quit()

    def on_state_changed(self, bus, msg):
        if msg.src != self._player:
            return

        print 'on_state_changed'
        old_state, new_state, pending = msg.parse_state_changed()
        print "%s -> %s" % (old_state, new_state)
        if old_state == Gst.State.READY and new_state == Gst.State.PAUSED:
            self.set_rate(self._rate)

    def set_rate(self, rate):
        self._rate = rate
        position = self._player.query_position(Gst.Format.TIME)[1]

        # Create the seek event
        seek_event = Gst.Event.new_seek(rate,
                                        Gst.Format.TIME,
                                        Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
                                        Gst.SeekType.SET, position,
                                        Gst.SeekType.SET, -1)

        if seek_event:
            self._player.send_event(seek_event)
            Gst.info("rate set to %s" % rate)
        else:
             Gst.warining("change rate failed")


def main(args):
    def usage():
        sys.stdout.write("usage: %s <filename> <speedrate>\n" % args[0])

    if len(args) != 3:
        usage()
        sys.exit(1)

    uri = Gst.filename_to_uri(args[1])
    rate = float(args[2])

    player = Player(uri, rate)
    player.run()

if __name__ == '__main__':
    sys.exit(main(sys.argv))
